from pytest_bdd import scenario, given, when, then, parsers
import re
import os
import pytest
from click.testing import CliRunner
from paylense.collections import Collections


pytest.globalDict = {}


@scenario('features/collections.feature', 'Request a payment from a consumer (Payer)')
def test_collections():
    pass


@given("I have a valid APP-ID, username and password")
def user_credentials():
    config = {
        "PAYLENSE_APP_ID": os.environ.get("PAYLENSE_APP_ID"),
        "PAYLENSE_USERNAME": os.environ.get("PAYLENSE_USERNAME"),
        "PAYLENSE_PASSWORD": os.environ.get("PAYLENSE_PASSWORD"),
    }
    client = Collections(config)
    pytest.globalDict["client"] = client


@when("I request for a payment with the following payment details\n| note         | amount | message | mobile     | product_id |\n| test payment | 600    | message | 0782631873 |")
def successful_request_to_pay():

    ref = pytest.globalDict["client"].requestToPay(
        mobile="256772123456", amount="600", processing_number="123456789", narrartion="narr")
    pytest.globalDict["ref"] = ref


@when("I check for transaction Status")
def check_transaction_status():
    status = pytest.globalDict["client"].getTransactionStatus(pytest.globalDict["ref"]["processing_number"])
    pytest.globalDict["status"] = status
    assert isinstance(status, dict)
    assert "amount" in status.keys()


@then("It should be successful")
def successful_transaction():
    assert pytest.globalDict["status"]["status"] == "SUCCESSFUL"



"""
Module that contains the command line app.
Why does this file exist, and why not put this in __main__?
  You might be tempted to import things from __main__ later, but that will cause
  problems: the code will get executed twice:
  - When you run `python -mpaylenseapi_python` python will execute
    ``__main__.py`` as a script. That means there won't be any
    ``paylenseapi_python.__main__`` in ``sys.modules``.
  - When you import __main__ it will get executed again (as a module) because
    there's no ``paylenseapi_python.__main__`` in ``sys.modules``.
  Also see (1) from http://click.pocoo.org/5/setuptools/#setuptools-integration
"""
import json
import time
import uuid

import click
import requests


def main():
    click.echo("Nothing..")





class PaylenseError(Exception):
    def __init__(self, message=None, http_body=None, http_status=None,
                 json_body=None, headers=None, code=None):
        super(PaylenseError, self).__init__(message)

        if http_body and hasattr(http_body, 'decode'):
            try:
                http_body = http_body.decode('utf-8')
            except BaseException:
                http_body = ('<Could not decode body as utf-8. ')

        self._message = message
        self.http_body = http_body
        self.http_status = http_status
        self.json_body = json_body
        self.headers = headers or {}
        self.code = code
        self.request_id = self.headers.get('request-id', None)

    def __str__(self):
        msg = self._message or "<empty message>"
        if self.request_id is not None:
            return u"Request {0}: {1}".format(self.request_id, msg)
        else:
            return msg

    # Returns the underlying `Exception` (base class) message

    @property
    def user_message(self):
        return self._message

    def __repr__(self):
        return '{0}(message={1}, http_status={2}, request_id={3})'.format(
            self.__class__.__name__,
            self._message,
            self.http_status,
            self.request_id)


class APIError(PaylenseError):
    pass


class APIConnectionError(PaylenseError):
    pass


class AuthenticationError(PaylenseError):
    pass


class PermissionError(PaylenseError):
    pass


class PreapprovalError(PaylenseError):
    pass


class RequestToPayError(PaylenseError):
    pass


class TransferError(PaylenseError):
    pass


class GeneralError(PaylenseError):
    pass


class ValidationError(Exception):
    pass


class ConfigurationError(Exception):
    pass



from pytest_bdd import scenario, given, when, then, parsers
import re
import os
import pytest
from click.testing import CliRunner
from paylense.disbursements import Disbursements

pytest.globalDict = {}


@scenario('features/disbursements.feature', 'Transfer Money to another account')
def test_disbursements():
    pass


@given("I have a valid APP-ID, username and password")
def user_credentials():
    config = {
        "PAYLENSE_APP_ID": os.environ.get("PAYLENSE_APP_ID"),
        "PAYLENSE_USERNAME": os.environ.get("PAYLENSE_USERNAME"),
        "PAYLENSE_PASSWORD": os.environ.get("PAYLENSE_PASSWORD"),
    }
    client = Disbursements(config)
    pytest.globalDict["client"] = client


@when("I transfer with the following payment details\n| note         | amount | message | mobile     | product_id |\n| test payment | 600    | message | 0782631873 |")
def successful_transfer():
    ref = pytest.globalDict["client"].transfer(
        amount="600", mobile="256772123456", processing_number="123456789", narration="dd")
    pytest.globalDict["ref"] = ref

    assert isinstance(ref, dict)
    assert "processing_number" in ref.keys()


@when("I check for transaction Status")
def check_transaction_status():
    status = pytest.globalDict["client"].getTransactionStatus(pytest.globalDict["ref"]["processing_number"])
    pytest.globalDict["status"] = status
    assert isinstance(status, dict)
    assert "amount" in status.keys()


@then("It should be successful")
def sucessful_transaction():
    assert pytest.globalDict["status"]["status"] == "SUCCESSFUL"



import requests
import unittest
try:
    from unittest import mock
except ImportError:
    import mock


class MockResponse:
    def __init__(self, json_data, status_code, headers={}):
        self.json_data = json_data
        self.status_code = status_code
        self.headers = headers

    def json(self):
        return self.json_data


def mocked_requests_get(*args, **kwargs):
    if "/collections" in args[0]:
        return MockResponse({
            "amount": 100,
            "msisdn": 4656473839,
            "processingNumber": 947354,
            "narration": "narr",
            "status": "SUCCESSFUL"
        }, 200)

    return MockResponse(None, 404)


def mocked_requests_post(*args, **kwargs):

    if "collections" in args[0]:
        return MockResponse({"key2": "value2"}, 200)

    return MockResponse(None, 404)


def mocked_requests_session(*args, **kwargs):
    if "/collections" in args[1] and args[0] == 'POST':
        return MockResponse({}, 200)
    elif "transfer" in args[1] and args[0] == 'POST':
        return MockResponse({}, 200)
    elif ("/collections" in args[1] or "/disbursements" in args[1]) and args[0] == 'GET':
        return MockResponse({
            "amount": 100,
            "msisdn": 4656473839,
            "processingNumber": 947354,
            "narration": "narr",
            "status": "SUCCESSFUL"
        }, 200)
    else:
        return MockResponse({}, 200)



import uuid

from .client import Paylense
from .utils import validate_phone_number


class Collections(Paylense, object):
    def getTransactionStatus(
            self,
            transaction_id,
            **kwargs):
        url = "/collections"

        return super(Collections, self).getTransactionStatus(
            transaction_id, url)

    def requestToPay(
            self,
            mobile,
            amount,
            processing_number,
            narration="",
            **kwargs):
            # type: (String,String,String,String,String,String,String) -> json
        ref = str(uuid.uuid4())
        data = {
            "msisdn": validate_phone_number(mobile),
            "amount": str(amount),
            "processingNumber": processing_number,
            "narration": narration}

        url = "{0}/collections".format(super(Collections, self).config.baseUrl)
        self.request("POST", url, headers, data)
        return {"transaction_ref": ref}



import unittest
import pytest
import types
try:
    from unittest import mock
except ImportError:
    import mock

from requests import Request, Session

from .utils import mocked_requests_get, mocked_requests_post, mocked_requests_session
from paylense.errors import ValidationError
from paylense.client import Paylense
from paylense.collections import Collections


class TestCollections(unittest.TestCase):

    @mock.patch('requests.post', side_effect=mocked_requests_post)
    def setUp(self, mock_get):
        self.config = {
            "PAYLENSE_APP_ID": "110000",
            "PAYLENSE_USERNAME": "sdk",
            "PAYLENSE_PASSWORD": "sdk@2020"
        }
        client = Collections(self.config)
        self.client = client

    def tearDown(self):
        pass

    @mock.patch('requests.get', side_effect=mocked_requests_get)
    def test_client_instantiate(self, mock_get):

        client = Collections(self.config)

        assert isinstance(client, Collections)

    @mock.patch('requests.get', side_effect=mocked_requests_get)
    def test_invalid_uuid(self, mock_get):
        with self.assertRaises(ValidationError):
            config = self.config
            client = Collections(config)

    @mock.patch('requests.get', side_effect=mocked_requests_get)
    def test_invalid_mobile(self, mock_get):
        with self.assertRaises(ValidationError):
            ref = self.client.requestToPay(mobile="256712123456", amount="600",
                                           processing_number="123456789", narration="dd")
        with self.assertRaises(ValidationError):
            ref = self.client.requestToPay(mobile="254712123456", amount="600",
                                           processing_number="123456789", narration="dd")

    @mock.patch.object(Paylense, "request", side_effect=mocked_requests_session)
    def test_request_to_pay(self, mock_get):

        ref = self.client.requestToPay(mobile="256772123456", amount="600", processing_number="123456789", narration="dd")

        assert isinstance(ref, dict)
        assert "processing_number" in ref.keys()

    @mock.patch.object(Paylense, "request", side_effect=mocked_requests_session)
    def test_get_transaction_status(self, mock_get):
        status = self.client.getTransactionStatus("dummy")
        assert isinstance(status, dict)
        assert "amount" in status.keys()



from .errors import ConfigurationError


class PaylenseConfig(object):

    def __init__(self, conf):
        """

        config={

            PAYLENSE_ENVIRONMENT: os.environ.get("PAYLENSE_ENVIRONMENT"),
            PAYLENSE_VERSION: os.environ.get("PAYLENSE_VERSION"),
            PAYLENSE_APP_ID: os.environ.get("PAYLENSE_APP_ID"),
            PAYLENSE_USERNAME: os.environ.get("PAYLENSE_USERNAME"),
            PAYLENSE_PASSWORD: os.environ.get("PAYLENSE_PASSWORD"),


        }


        """
        self._config = conf

    def get_property(self, property_name):
        if property_name not in self._config.keys():
            return None
        return self._config[property_name]

    @property
    def environment(self):
        return self.get_property('PAYLENSE_ENVIRONMENT') or "sandbox"

    @property
    def version(self):
        return self.get_property('PAYLENSE_VERSION') or "v1"

    @property
    def baseUrl(self):
        if self.environment == "sandbox":
            return "https://api-sandbox.paylense.com/" + self.version
        else:
            return "https://api.paylense.com/" + self.version

    @property
    def app_id(self):
        key = self.get_property('PAYLENSE_APP_ID')
        if not key:
            raise ConfigurationError(
                "PAYLENSE_APP_ID is missing in the configuration")
        else:
            key

    @property
    def username(self):
        key = self.get_property('PAYLENSE_USERNAME')
        if not key:
            raise ConfigurationError(
                "PAYLENSE_USERNAME is missing in the configuration")
        else:
            key

    @property
    def password(self):
        key = self.get_property('PAYLENSE_PASSWORD')
        if not key:
            raise ConfigurationError(
                "PAYLENSE_PASSWORD is missing in the configuration")
        else:
            return key



from uuid import UUID
import phonenumbers
from phonenumbers import carrier

import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry
from .errors import ValidationError

ERROR_CODES = [
    {
        "http_code": 409,
        "response_code": None,
        "description": "Duplicated Processing number. Cannot create new resource",
        "error_type": "generic"

    },
    {
        "http_code": 404,
        "response_code": None,
        "description": "Processing number not found. Requested resource does not exist",
        "error_type": "generic"

    },
    {
        "http_code": 400,
        "response_code": None,
        "description": "Bad request. Request does not follow the specification.",
        "error_type": "generic"

    },
    {
        "http_code": 401,
        "response_code": None,
        "description": "Authentication failed. Credentials not valid",
        "error_type": "generic"

    },

    {
        "http_code": 500,
        "response_code": "NOT_ALLOWED",
        "description": "Authorization failed. User does not have permission.",
        "error_type": "generic"

    },

    {
        "http_code": 500,
        "response_code": "INTERNAL_PROCESSING_ERROR",
        "description": "Default error code used when there is no specific error mapping.",
        "error_type": "generic"

    },

    {
        "http_code": 500,
        "response_code": "SERVICE_UNAVAILABLE",
        "description": "Service temporary unavailable, try again later",
        "error_type": "generic"

    },

    {
        "http_code": 404,
        "response_code": None,
        "description": "Account holder is not found",
        "error_type": "account"

    }
]


def requests_retry_session(
    retries=3,
    backoff_factor=0.3,
    status_forcelist=(502, 504),
    session=None,
    **kwargs
):
    session = session or requests.Session()
    retry = Retry(
        total=retries,
        read=retries,
        connect=retries,
        backoff_factor=backoff_factor,
        status_forcelist=status_forcelist,
    )
    adapter = HTTPAdapter(max_retries=retry)
    session.mount('http://', adapter)
    session.mount('https://', adapter)
    return session


def validate_phone_number(number):
    obj = phonenumbers.parse(number, "UG")
    if not phonenumbers.is_valid_number(obj):
        raise ValidationError("Invalid Phone number {0}".format(number))
    if (carrier.name_for_number(obj, "en") != "MTN") or (carrier.name_for_number(obj, "en") != "Airtel"):
        raise ValidationError(
            "{0}: Only MTN | Airtel is supported at the moment".format(number))
    return "256{0}".format(obj.national_number)


def validate_number(number):
    number_types = (int, float)
    if not type(number) in number_types:
        raise ValidationError("{0}: Must be a number".format(number))
    return number


def validate_string(_string):
    if not isinstance(_string, str):
        raise ValidationError("{0}: Must be a string".format(_string))
    return _string


def validate_uuid(_string):
    try:
        UUID(_string, version=4)
    except ValueError:
        raise ValidationError(
            "{0}: Must be a valid uuid4 string".format(_string))
    return _string



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function

import io
import re
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

from setuptools import find_packages
from setuptools import setup


def read(*names, **kwargs):
    with io.open(
        join(dirname(__file__), *names),
        encoding=kwargs.get('encoding', 'utf8')
    ) as fh:
        return fh.read()


setup(
    name='paylense-sdk',
    version='1.0.1',
    license='MIT license',
    description='Python wrapper for the Winopay Paylense API.',
    long_description='%s\n%s' % (
        re.compile('^.. start-badges.*^.. end-badges', re.M | re.S).sub('', read('README.md')),
        re.sub(':[a-z]+:`~?(.*?)`', r'``\1``', read('CHANGELOG.md'))
    ),
    long_description_content_type='text/markdown',
    author='Winopay',
    author_email='acellam.guy@gmail.com',
    url='https://github.com/winopay/paylense-python-sdk',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    py_modules=[splitext(basename(path))[0] for path in glob('src/*.py')],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Utilities',
    ],
    keywords=[
        'Paylense API', 'Paylense API Python Wrapper','Paylense API Python', 'Paylense API'
    ],
    install_requires=[
        'requests == 2.21.0',
        'Click==7.0',
        'phonenumbers'

        # eg: 'aspectlib==1.1.1', 'six>=1.7',
    ],

    setup_requires=["pytest-runner", "pytest-cov"],


    extras_require={'test': ['pytest', 'pytest-watch', 'tox',
                             'pytest-cov',
                             'pytest-pep8',
                             'pytest-cov',
                             'pytest-sugar',
                             'mock',
                             'pytest-instafail',
                             'pytest-bdd'], "dev": ["semver"]},
    entry_points={
        'console_scripts': [
            'paylense = paylense.cli:main',
        ]
    },
)



"""
Main client class

@author: Acellam Guy
"""
import json
try:
    from json.decoder import JSONDecodeError
except ImportError:
    JSONDecodeError = ValueError

import requests
from requests import Request, Session
from requests._internal_utils import to_native_string
from requests.auth import AuthBase
from requests.auth import HTTPBasicAuth


from .config import PaylenseConfig
from .errors import APIError
from .utils import requests_retry_session


class Response:

    def __init__(self, body, code, headers):
        self.body = body
        self.code = code
        self.headers = headers
        self.data = body


class ClientInterface():
    def getTransactionStatus(self):
        raise NotImplementedError


class Client(ClientInterface):
    def getTransactionStatus(self):
        return super(Client, self).getTransactionStatus()


class Paylense(ClientInterface, object):

    def __init__(
            self,
            config,
            ** kwargs):
        super(Paylense, self).__init__(**kwargs)
        self._session = Session()
        self._config = PaylenseConfig(config)

    @property
    def config(self):
        return self._config

    def request(self, method, url, post_data=None):
        headers = {
            "Content-Type": "application/json",
            "APP-ID": self.config.app_id
        }
        request = Request(
            method,
            url,
            data=json.dumps(post_data),
            headers=headers,
            auth=HTTPBasicAuth(
                self.config.username,
                self.config.password))

        prepped = self._session.prepare_request(request)

        resp = requests_retry_session(sesssion=self._session).send(prepped,
                                                                   verify=False
                                                                   )
        return self.interpret_response(resp)

    def interpret_response(self, resp):
        rcode = resp.status_code
        rheaders = resp.headers
        print(resp)

        try:
            rbody = resp.json()
        except JSONDecodeError:
            rbody = resp.text
            resp = Response(rbody, rcode, rheaders)

        if not (200 <= rcode < 300):
            self.handle_error_response(rbody, rcode, resp.text, rheaders)

        return resp

    def handle_error_response(self, rbody, rcode, resp, rheaders):

        raise APIError(
            "Invalid response object from API: {0} (HTTP response code "
            "was {1})".format(rbody, rcode),
            rbody, rcode, resp)

    def request_headers(self, api_key, method):
        headers = {}

        return headers

    def getTransactionStatus(
            self,
            transaction_id,
            url,
            ** kwargs):

        _url = self.config.baseUrl + url + transaction_id
        print(_url)
        res = self.request("GET", _url)
        return res.json()

    def close(self):
        if self._session is not None:
            print("closing!")
            self._session.close()
